@@ -43,7 +43,13 @@ This module contains a document language model for VHDL.
 # load dependencies
 from enum               import Enum
 from pathlib            import Path
-from typing             import List, Tuple, Union
+from typing import List, Tuple, Union, Dict
+
+try:
+	from typing import Protocol
+except ImportError:
+	class Protocol:
+		pass
 
 from pydecor.decorators import export
 
@@ -251,6 +257,19 @@ class ArchitectureSymbol(Symbol):
 		return self._architecture
 
 
+@export
+class ComponentSymbol(Symbol):
+	_component: 'Component'
+
+	def __init__(self):
+		super().__init__()
+		self._component = None
+
+	@property
+	def Component(self) -> 'Component':
+		return self._component
+
+
 @export
 class ConfigurationSymbol(Symbol):
 	_configuration: 'Configuration'
@@ -307,7 +326,21 @@ class SimpleSubTypeSymbol(SubTypeSymbol):
 
 
 @export
-class ConstrainedSubTypeSymbol(SubTypeSymbol):
+class ConstrainedScalarSubTypeSymbol(SubTypeSymbol):
+	_range: 'Range'
+
+	def __init__(self, subTypeName: str, range: 'Range' = None):
+		super().__init__(symbolName = subTypeName)
+		self._subType = None
+		self._range = range
+
+	@property
+	def Range(self) -> 'Range':
+		return self._range
+
+
+@export
+class ConstrainedCompositeSubTypeSymbol(SubTypeSymbol):
 	_constraints: List[Constraint]
 
 	def __init__(self, subTypeName: str, constraints: List[Constraint] = None):
@@ -329,16 +362,35 @@ class EnumerationLiteralSymbol(Symbol):
 class ObjectSymbol(Symbol):
 	pass
 
+
 @export
-class SimpleObjectSymbol(Symbol):
-	_object: Union['Constant', 'Signal', 'Variable']
+class SimpleObjectOrFunctionCallSymbol(Symbol):
+	_object: Union['Constant', 'Signal', 'Variable', 'Function']
 
 	def __init__(self, objectName: str):
 		super().__init__(objectName)
 		self._object = None
 
 	@property
-	def Object(self) -> Union['Constant', 'Signal', 'Variable']:
+	def Object(self) -> Union['Constant', 'Signal', 'Variable', 'Function']:
+		return self._object
+
+	def __str__(self) -> str:
+		if self._object is not None:
+			return str(self._object)
+		return super().__str__()
+
+
+@export
+class IndexedObjectOrFunctionCallSymbol(Symbol):
+	_object: Union['Constant', 'Signal', 'Variable', 'Function']
+
+	def __init__(self, objectName: str):
+		super().__init__(objectName)
+		self._object = None
+
+	@property
+	def Object(self) -> Union['Constant', 'Signal', 'Variable', 'Function']:
 		return self._object
 
 	def __str__(self) -> str:
@@ -406,17 +458,17 @@ class Design(ModelEntity):
 	and analysed. It's the root of this document-object-model (DOM). It contains
 	at least on VHDL library (see :class:`~pyVHDLModel.VHDLModel.Library`).
 	"""
-	_libraries:  List['Library']  #: List of all libraries defined for a design.
+	_libraries:  Dict[str, 'Library']  #: List of all libraries defined for a design.
 	_documents:  List['Document'] #: List of all documents loaded for a design.
 
 	def __init__(self):
 		super().__init__()
 
-		self._libraries = []
+		self._libraries = {}
 		self._documents = []
 
 	@property
-	def Libraries(self) -> List['Library']:
+	def Libraries(self) -> Dict[str, 'Library']:
 		"""Returns a list of all libraries specified for this design."""
 		return self._libraries
 
@@ -425,6 +477,30 @@ class Design(ModelEntity):
 		"""Returns a list of all documents (files) loaded for this design."""
 		return self._documents
 
+	def GetLibrary(self, libraryName: str) -> 'Library':
+		if libraryName not in self._libraries:
+			lib = Library(libraryName)
+			self._libraries[libraryName] = lib
+		else:
+			lib = self._libraries[libraryName]
+
+		return lib
+
+	def AddDocument(self, document: 'Document', library: 'Library') -> None:
+		self._documents.append(document)
+
+		for entity in document.Entities:
+			library.Entities.append(entity)
+
+		for package in document.Packages:
+			library.Packages.append(package)
+
+		for configuration in document.Configurations:
+			library.Configurations.append(configuration)
+
+		for context in document.Contexts:
+			library.Contexts.append(context)
+
 
 @export
 class Library(ModelEntity, NamedEntity):
@@ -527,6 +603,18 @@ class Document(ModelEntity):
 		return self._packageBodies
 
 
+@export
+class Alias(ModelEntity, NamedEntity):
+	def __init__(self, name: str):
+		"""
+		Initializes underlying ``BaseType``.
+
+		:param name: Name of the type.
+		"""
+		super().__init__()
+		NamedEntity.__init__(self, name)
+
+
 @export
 class BaseType(ModelEntity, NamedEntity):
 	"""``BaseType`` is the base class of all type entities in this model."""
@@ -573,7 +661,7 @@ class SubType(BaseType):
 
 
 @export
-class ScalarType(BaseType):
+class ScalarType(Type):
 	"""
 	A ``ScalarType`` is a base-class for all scalar types.
 	"""
@@ -612,39 +700,55 @@ class DiscreteType:
 
 
 @export
-class CompositeType(BaseType):
+class CompositeType(Type):
 	"""
 	A ``CompositeType`` is a base-class for all composite types.
 	"""
 
 
 @export
-class ProtectedType(BaseType):
+class ProtectedType(Type):
 	pass
 
 
 @export
-class AccessType(BaseType):
-	pass
+class AccessType(Type):
+	_designatedSubType: SubTypeOrSymbol
+
+	def __init__(self, name: str, designatedSubType: SubTypeOrSymbol):
+		super().__init__(name)
+		self._designatedSubType = designatedSubType
+
+	@property
+	def DesignatedSubtype(self):
+		return self._designatedSubType
 
 
 @export
-class FileType(BaseType):
-	pass
+class FileType(Type):
+	_designatedSubType: SubTypeOrSymbol
+
+	def __init__(self, name: str, designatedSubType: SubTypeOrSymbol):
+		super().__init__(name)
+		self._designatedSubType = designatedSubType
+
+	@property
+	def DesignatedSubtype(self):
+		return self._designatedSubType
 
 
 @export
 class EnumeratedType(ScalarType, DiscreteType):
-	_elements: List['EnumerationLiteral']
+	_literals: List['EnumerationLiteral']
 
-	def __init__(self, name: str):
+	def __init__(self, name: str, literals: List['EnumerationLiteral']):
 		super().__init__(name)
 
-		self._elements = []
+		self._literals = [] if literals is None else [l for l in literals]
 
 	@property
-	def Elements(self) -> List['EnumerationLiteral']:
-		return self._elements
+	def Literals(self) -> List['EnumerationLiteral']:
+		return self._literals
 
 
 @export
@@ -683,7 +787,7 @@ class ArrayType(CompositeType):
 	_dimensions:  List['Range']
 	_elementType: SubType
 
-	def __init__(self, name: str):
+	def __init__(self, name: str, indices: List, elementSubType: SubTypeOrSymbol):
 		super().__init__(name)
 
 		self._dimensions =  []
@@ -698,33 +802,37 @@ class ArrayType(CompositeType):
 
 
 @export
-class RecordTypeMember(ModelEntity):
+class RecordTypeElement(ModelEntity):
 	_name:    str
-	_subType: SubType
+	_subType: SubTypeOrSymbol
 
-	def __init__(self, name: str):
+	def __init__(self, name: str, subType: SubTypeOrSymbol):
 		super().__init__()
 
-		self._name =        name
-		self._subType =     None
+		self._name =    name
+		self._subType = subType
 
 	@property
 	def Name(self) -> str:
 		return self._name
 
+	@property
+	def SubType(self) -> SubTypeOrSymbol:
+		return self._subType
+
 
 @export
 class RecordType(CompositeType):
-	_members: List[RecordTypeMember]
+	_elements: List[RecordTypeElement]
 
-	def __init__(self, name: str):
+	def __init__(self, name: str, elements: List[RecordTypeElement] = None):
 		super().__init__(name)
 
-		self._members =     []
+		self._elements = [] if elements is None else [i for i in elements]
 
 	@property
-	def Members(self) -> List[RecordTypeMember]:
-		return self._members
+	def Elements(self) -> List[RecordTypeElement]:
+		return self._elements
 
 
 @export
@@ -798,7 +906,45 @@ class FloatingPointLiteral(NumericLiteral):
 
 @export
 class PhysicalLiteral(NumericLiteral):
-	pass
+	_unitName: str
+
+	def __init__(self, unitName: str):
+		super().__init__()
+		self._unitName = unitName
+
+	@property
+	def UnitName(self) -> str:
+		return self._unitName
+
+	def __str__(self) -> str:
+		return "{value} {unit}".format(value=self._value, unit=self._unitName)
+
+
+@export
+class PhysicalIntegerLiteral(PhysicalLiteral):
+	_value: int
+	_unitName: str
+
+	def __init__(self, value: int, unitName: str):
+		super().__init__(unitName)
+		self._value = value
+
+	@property
+	def Value(self) -> int:
+		return self._value
+
+
+@export
+class PhysicalFloatingLiteral(PhysicalLiteral):
+	_value: float
+
+	def __init__(self, value: float, unitName: str):
+		super().__init__(unitName)
+		self._value = value
+
+	@property
+	def Value(self) -> float:
+		return self._value
 
 
 @export
@@ -849,6 +995,13 @@ class BitStringLiteral(Literal):
 		return "\"" + self._value + "\""
 
 
+@export
+class ParenthesisExpression(Protocol):
+	@property
+	def Operand(self) -> Expression:
+		pass
+
+
 @export
 class UnaryExpression(BaseExpression):
 	"""
@@ -873,7 +1026,7 @@ class UnaryExpression(BaseExpression):
 		)
 
 @export
-class InverseExpression(UnaryExpression):
+class NegationExpression(UnaryExpression):
 	_FORMAT = ("-", "")
 
 @export
@@ -881,7 +1034,7 @@ class IdentityExpression(UnaryExpression):
 	_FORMAT = ("+", "")
 
 @export
-class NegationExpression(UnaryExpression):
+class InverseExpression(UnaryExpression):
 	_FORMAT = ("not ", "")
 
 @export
@@ -896,8 +1049,9 @@ class TypeConversion(UnaryExpression):
 class FunctionCall(UnaryExpression):
 	pass
 
+
 @export
-class ParenthesisExpression(UnaryExpression):
+class SubExpression(UnaryExpression, ParenthesisExpression):
 	_FORMAT = ("(", ")")
 
 
@@ -932,10 +1086,6 @@ class BinaryExpression(BaseExpression):
 		)
 
 
-@export
-class QualifiedExpression(BinaryExpression):
-	pass
-
 @export
 class	AddingExpression(BinaryExpression):
 	"""
@@ -1083,6 +1233,30 @@ class	RotateRightExpression(RotateExpression):
 class	RotateLeftExpression(RotateExpression):
 	_FORMAT = ("", " rol ", "")
 
+
+@export
+class QualifiedExpression(BaseExpression, ParenthesisExpression):
+	_operand:  Expression
+	_subtype:  SubTypeOrSymbol
+
+	def __init__(self):
+		super().__init__()
+
+	@property
+	def Operand(self):
+		return self._operand
+
+	@property
+	def SubTyped(self):
+		return self._subtype
+
+	def __str__(self) -> str:
+		return "{subtype}'({operand!s})".format(
+			subtype=self._subtype,
+			operand=self._operand,
+		)
+
+
 @export
 class TernaryExpression(BaseExpression):
 	"""
@@ -1207,7 +1381,7 @@ class Aggregate(BaseExpression):
 		return self._elements
 
 	def __str__(self) -> str:
-		choices = [self.formatAggregateElement(element) for element in self._elements]
+		choices = [str(element) for element in self._elements]
 		return "({choices})".format(
 			choices=", ".join(choices)
 		)
@@ -1313,6 +1487,11 @@ class Variable(Object, WithDefaultExpression):
 	pass
 
 
+@export
+class SharedVariable(Object):
+	pass
+
+
 @export
 class Signal(Object, WithDefaultExpression):
 	pass
@@ -1654,19 +1833,19 @@ class Context(PrimaryUnit):
 
 @export
 class Entity(PrimaryUnit, MixinDesignUnitWithContext):
-	_genericItems:      List[GenericInterfaceItem]
-	_portItems:         List[PortInterfaceItem]
-	_declaredItems:     List   # FIXME: define list element type e.g. via Union
-	_bodyItems:         List['ConcurrentStatement']
+	_genericItems:  List[GenericInterfaceItem]
+	_portItems:     List[PortInterfaceItem]
+	_declaredItems: List   # FIXME: define list element type e.g. via Union
+	_bodyItems:     List['ConcurrentStatement']
 
-	def __init__(self, name: str):
+	def __init__(self, name: str, genericItems: List[GenericInterfaceItem] = None, portItems: List[PortInterfaceItem] = None, declaredItems: List = None, bodyItems: List['ConcurrentStatement'] = None):
 		super().__init__(name)
 		MixinDesignUnitWithContext.__init__(self)
 
-		self._genericItems      = []
-		self._portItems         = []
-		self._declaredItems     = []
-		self._bodyItems         = []
+		self._genericItems  = [] if genericItems is None else [g for g in genericItems]
+		self._portItems     = [] if portItems is None else [p for p in portItems]
+		self._declaredItems = [] if declaredItems is None else [i for i in declaredItems]
+		self._bodyItems     = [] if bodyItems is None else [i for i in bodyItems]
 
 	@property
 	def GenericItems(self) -> List[GenericInterfaceItem]:
@@ -1687,19 +1866,20 @@ class Entity(PrimaryUnit, MixinDesignUnitWithContext):
 
 @export
 class Architecture(SecondaryUnit, MixinDesignUnitWithContext):
-	_entity:            Entity
-	_declaredItems:     List   # FIXME: define list element type e.g. via Union
-	_bodyItems:         List['ConcurrentStatement']
+	_entity:        EntityOrSymbol
+	_declaredItems: List   # FIXME: define list element type e.g. via Union
+	_bodyItems:     List['ConcurrentStatement']
 
-	def __init__(self, name: str):
+	def __init__(self, name: str, entity: EntityOrSymbol, declaredItems: List = None, bodyItems: List['ConcurrentStatement'] = None):
 		super().__init__(name)
 		MixinDesignUnitWithContext.__init__(self)
 
-		self._declaredItems =     []
-		self._bodyItems =         []
+		self._entity        = entity
+		self._declaredItems = [] if declaredItems is None else [i for i in declaredItems]
+		self._bodyItems     = [] if bodyItems is None else [i for i in bodyItems]
 
 	@property
-	def Entity(self) -> Entity:
+	def Entity(self) -> EntityOrSymbol:
 		return self._entity
 
 	@property
@@ -1711,6 +1891,27 @@ class Architecture(SecondaryUnit, MixinDesignUnitWithContext):
 		return self._bodyItems
 
 
+@export
+class Component(ModelEntity, NamedEntity):
+	_genericItems:      List[GenericInterfaceItem]
+	_portItems:         List[PortInterfaceItem]
+
+	def __init__(self, name: str, genericItems: List[GenericInterfaceItem] = None, portItems: List[PortInterfaceItem] = None):
+		super().__init__()
+		NamedEntity.__init__(self, name)
+
+		self._genericItems      = [] if genericItems is None else [g for g in genericItems]
+		self._portItems         = [] if portItems is None else [p for p in portItems]
+
+	@property
+	def GenericItems(self) -> List[GenericInterfaceItem]:
+		return self._genericItems
+
+	@property
+	def PortItems(self) -> List[PortInterfaceItem]:
+		return self._portItems
+
+
 @export
 class Configuration(PrimaryUnit, MixinDesignUnitWithContext):
 	def __init__(self, name: str):
@@ -1778,12 +1979,12 @@ class Package(PrimaryUnit, MixinDesignUnitWithContext):
 	_genericItems:      List[GenericInterfaceItem]
 	_declaredItems:     List
 
-	def __init__(self, name: str):
+	def __init__(self, name: str, genericItems: List[GenericInterfaceItem] = None, declaredItems: List = None):
 		super().__init__(name)
 		MixinDesignUnitWithContext.__init__(self)
 
-		self._genericItems =      []
-		self._declaredItems =     []
+		self._genericItems =  [] if genericItems is None else [g for g in genericItems]
+		self._declaredItems = [] if declaredItems is None else [i for i in declaredItems]
 
 	@property
 	def GenericItems(self) -> List[GenericInterfaceItem]:
@@ -1799,11 +2000,11 @@ class PackageBody(SecondaryUnit, MixinDesignUnitWithContext):
 	_package:           Package
 	_declaredItems:     List
 
-	def __init__(self, name: str):
+	def __init__(self, name: str, declaredItems: List = None):
 		super().__init__(name)
 		MixinDesignUnitWithContext.__init__(self)
 
-		self._declaredItems =     []
+		self._declaredItems = [] if declaredItems is None else [i for i in declaredItems]
 
 	@property
 	def Package(self) -> Package:
